# copybook_tsoc.pl - Top script called to upload copybook to mainframe from TSO Client
#
# Authors:  Jane Huang
# Last updated:  3/22/07
#      Xiao Lin Zhang 28/6/10
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is the top of the call sequence for uploading copybook member
# to mainframe.
#
# This script is called thru TSO Client.
# The "Upload Copybook to Host" function on TSO Client Panel 2.2 will perform the upload file.
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 

use File::Basename;
use File::Copy;
use Getopt::Std;
use bccpbmod;

my $debug = 0;

# #############################################################################
# Use getopt to process the command arguments
#
# "-d dir"   identifies the directory where the elements are located
# "file ..." identifies the file or list of files to be uploaded
#

my $usage = "copybook_tsoc.pl -d dir file ...";

my $upload_1 = "";
my $elem_dir  = ""; # directory where the file resides

  if (!getopts('d:') || $opt_h) {
    print "$usage \n";
    exit(0);
}


$elem_dir  = $opt_d;

my $current_dir = `cd`;

print "Main Copybook for TSOC: current dir =  $current_dir \n" if $debug;
print "Main Copybook: dir_file = $elem_dir \n" if $debug;

# #############################################################################
# Initialize variable for various executables and scripts:
#
# These variable must be modified for each installation
# #############################################################################
# *******************************************************************************
# * Required customization:                                                     *
# * The location of the PERL path                                               *
# *******************************************************************************

#my $perlexe	 = "C:\\Program Files\\Rational\\ClearCase\\bin\\ccperl.exe";
my $perlexe	 = "ccperl";


# *******************************************************************************
# * Required customization:                                                     *
# * The location of the build script can be set with a type attribute SCRIPTDIR *
# * in PVOB or hardcoded here.                                                  *
# *******************************************************************************

my $scriptdir = "";   # location of the build scripts
my $uploadcopy= "";   # Upload CopyBook script 
my $project   = "";

  
  $project = `cleartool lsstream -fmt \"%[project]Xp\"`;       
  $scriptdir = `cleartool desc -fmt \"%[SCRIPTDIR]SNa\" $project`;


  # printf( "Script dir set by type attribute SCRIPTDIR:(%s)\n", $scriptdir ) if $debug;    

     
    if ($scriptdir) {
   	$scriptdir =~ s/^\"//;  # remove quotes around attr value
   	$scriptdir =~ s/\"$//;  # returned from cleartool cmd
         
        $uploadcopy = $scriptdir ."\\uploadCopybook.pl";         
            
        printf( "Build script dir by attribute SCRIPTDIR:(%s)\n", $scriptdir ) if $debug; 
        printf( "Upload Copybook script:(%s)\n", $uploadcopy ) if $debug;  

    } else {      
              
        $scriptdir = "\\\\PFHUANG\\ccstg_d\\Triggers";
        $uploadcopy= "\\\\PFHUANG\\ccstg_d\\Triggers\\uploadCopybook.pl";
        printf( "Build script dir by hardcoded:(%s)\n", $scriptdir ) if $debug;  
    }
     
    
    # check if $uploadcopy = "";
    if ($scriptdir eq "" || $uploadcopy eq "") {
      	print " Define the build script directory and upload copybook script !\n" ;
        exit(0);
    }
  

print "Upload script for TSOC Current Directory " .$elem_dir. "\n\n";

# #############################################################################
# check for multiple files to upload and set a variable to be used for
# the upload scripts
# 
my $num_args = @ARGV;
my $upload_1 = "";
# $upload_1 = "-1" if ($num_args == 1);

# #############################################################################
# Get the environment variables from the common module
# These variables establish the distributed and mainframe environment
# and must be modified for each installation.  The values for these 
# variables are held in attributes attached to any/every Z-enabled
# ClearCase UCM Project.
# 
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.  If you determine to use the optional variables or add new variables,*
# * you have to update all related scripts correspondently.                     *
# *******************************************************************************
my ($env, $db2sys, $menv, $os390_hostname, $os390_port, $stage,
    $copylib1, $copylib2, $copylib3, $copylib4,
    $syslib1, $syslib2, $syslib3,
#    $syslib4, $steplib1, $steplib2,			 # Optional
#    $hostpds_prefix, $hostpds_affix, $cbllib,	         # Optional 
    $loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) =
  bccpbmod::setEnvironmentVar();

my $pcid = $ENV{USERNAME};
my $mfid = bccpbmod::getUserID($pcid);
bccpbmod::my_die('Unknown User.') if ($mfid eq "");

# #############################################################################
# Identify each of the file types and separate files from the arguments list
# into arrays by type.  These arrays are passed to the type specific uploading
# scripts.
# 

my @cobol2_files;
my $no_cobol2_files = 0;

my $no_unsupported_files = 0;
my $message ="";  # test only -add


foreach $path (@ARGV) {
	my %CCattr = bccpbmod::getCCattrs($path);

	print "path=$path LANG=$CCattr{'LANG'}\n" if $debug;


		push(@cobol2_files, $path);

}
 
# #############################################################################
# If any of the file type arrays contain filenames, then execute the uploading
# script for each file in the list.
# 

$no_cobol2_files = @cobol2_files;


# ------------------------- Upload CopyBook files -----------------------------
if ($no_cobol2_files != 0) {
	print "Uploading Copybook files\n";
	bccpbmod::printArray(@cobol2_files);

	foreach $path (@cobol2_files) { 
		my($file, $directory, $suffix) = fileparse($path);   
       		
		print "======================================================= \n";
		print " Uploading file: " .$file. "\n";
    

# #############################################################################  		
# Exit if the file is not a COPYBOOK     
#
    
   my($testfile,$testdir,$ext) = File::Basename::fileparse($file,qr{.cpy});  
     
    if (($ext ne ".cpy") && ($ext ne ".CPY")) {
      	print "ERROR- File extension $ext is not correct !\n" ;
   	exit -1;
   }

# #############################################################################
# Upload the copybook file
#
    	my $delimiter= '\\';
			my $del_ind  = rindex( $file, $delimiter);
			my $mem_ext  = substr( $file, ($del_ind == -1 ? 0: ($del_ind+1)), length($file));
			my $dot      = rindex( $mem_ext, '.' );
			my $mem      = substr( $mem_ext, 0, $dot );

    	my $jcldir = $elem_dir.$delimiter.$mfid;
			if(-d $jcldir){}
			else{mkdir($jcldir);}

			$jcldir = $jcldir.$delimiter.$mem;
			if(-d $jcldir){}
			else{mkdir($jcldir);}
			
			# Determine the directory context for the execution
			if ($jcldir ne '') {
    			# directory of file specified, cd to there
    		print "Changing directory from $current_dir to $jcldir\n" if $debug;
    		chdir($jcldir);

    		$current_dir = `cd`;
    		print "current dir =  $current_dir \n" if $debug;
			}		
			
			# #############################################################################
			# Check for connectivity to the mainframe and die if not successful.
			bccpbmod::my_die("Server down error.") if (bccpbmod::testServer($os390_hostname, $os390_port) != 0);

		my $logfile = "";
		
     		$logfile = $jcldir . $delimiter. $mfid .'_'. $mem . "_uploadCopybook.log";
   		print $logfile . "\n";


                `ccperl -I\"$scriptdir\" \"$uploadcopy\" $upload_1 -d \"$elem_dir\" -f \"$file\" > \"$logfile\"`;  

		if ($?) {
	        	exit -1;
		}

		print "\n-------------------------------------------------------- \n";
		print "Completed uploading copybook member of $file\n";
		print "-------------------------------------------------------- \n\n";
	}
}


#------------------------------------------------------
#
# Wait at the end of the upload process for the user to press "any key".  This allows the user to
# review the DOS window output before the window is closed.
# 

exit 0;

#######################################################
